@@ -19,7 +19,6 @@ import argparse
 import atexit
 
 import builtins
-import contextlib
 import dis
 import os
 import platform
@@ -31,7 +30,6 @@ import threading
 import time
 import traceback
 from collections import defaultdict
-from contextlib import contextmanager
 from functools import lru_cache
 from rich.console import Console
 from rich.markdown import Markdown
@@ -47,32 +45,22 @@ from typing import (
     Callable,
     Dict,
     FrozenSet,
-    IO,
-    Iterator,
     List,
     NewType,
     Optional,
     Set,
     Tuple,
-    TypeVar,
     Union,
     cast,
 )
 
-# from . import adaptive
-# from . import sparkline
-
 
 class adaptive:
     """Implements sampling to achieve the effect of a uniform random sample."""
 
     sample_array: List[float] = []
     current_index = 0
-    total_samples = 0
     max_samples = 0
-    count_average = 0
-    sum_average = 0
-    max_average = 1
 
     def __init__(self, size: int):
         self.max_samples = size
@@ -89,10 +77,8 @@ class adaptive:
                 new_array[i] = arr[1]  # Median
             self.current_index = self.max_samples // 3
             self.sample_array = new_array
-            # Update average length
-            self.max_average *= 3
         self.sample_array[self.current_index] = value
-        self.current_index += 1  # count_average += 1
+        self.current_index += 1
 
     def get(self) -> List[float]:
         return self.sample_array
@@ -132,12 +118,9 @@ def sparkline(
         mx = float(max(numbers))
     else:
         mx = fixed_max
-    # print(numbers)
-    # mn, mx = min(numbers), max(numbers)
     extent = mx - mn
     if extent == 0:
         extent = 1
-    # print("mn, mx = " + str(mn) + ", " + str(mx) + " extent = " + str(extent))
     sparkstr = "".join(
         bar[min([barcount - 1, int((n - mn) / extent * barcount)])] for n in numbers
     )
@@ -184,7 +167,6 @@ def parse_args() -> Tuple[argparse.Namespace, List[str]]:
         formatter_class=argparse.RawTextHelpFormatter,
         allow_abbrev=False,
     )
-    parser.add_argument("prog", type=str, help="program to be profiled")
     parser.add_argument(
         "--outfile",
         type=str,
@@ -221,6 +203,21 @@ def parse_args() -> Tuple[argparse.Namespace, List[str]]:
         default=False,
         help="only profile CPU time (default: profile CPU, memory, and copying)",
     )
+    parser.add_argument(
+        "--profile-all",
+        dest="profile_all",
+        action="store_const",
+        const=True,
+        default=False,
+        help="profile all executed code, not just the target program (default: only the target program)",
+    )
+    parser.add_argument(
+        "--cpu-percent-threshold",
+        dest="cpu_percent_threshold",
+        type=int,
+        default=1,
+        help="only report profiles with at least this percent of CPU time (default: 1%%)",
+    )
     # the PID of the profiling process (for internal use only)
     parser.add_argument(
         "--pid", type=int, default=int(os.getpid()), help=argparse.SUPPRESS
@@ -267,7 +264,6 @@ if (
             "PYTHONMALLOC" not in os.environ
         ):
             env = os.environ
-            # qenv = {}
             env["DYLD_INSERT_LIBRARIES"] = os.path.join(
                 os.path.dirname(__file__), "libscalene.dylib"
             )
@@ -285,6 +281,12 @@ ByteCodeIndex = NewType("ByteCodeIndex", int)
 class Scalene:
     """The Scalene profiler itself."""
 
+    # Debugging flag, for internal use only.
+    __debug = False
+
+    # Hijack OS open calls.
+    __hijack_open_calls = False  # disabled for now
+
     # We use these in is_call_function to determine whether a
     # particular bytecode is a function call.  We use this to
     # distinguish between Python and native code execution when
@@ -391,6 +393,12 @@ class Scalene:
         lambda: defaultdict(float)
     )
 
+    # Running count of total CPU samples per file. Used to prune reporting.
+    __cpu_samples: Dict[Filename, float] = defaultdict(float)
+
+    # Running count of malloc samples per file. Used to prune reporting.
+    __malloc_samples: Dict[Filename, float] = defaultdict(float)
+
     # Below are indexed by [filename][line_no][bytecode_index]:
     #
 
@@ -456,14 +464,14 @@ class Scalene:
         lambda: defaultdict(lambda: adaptive(9))
     )
 
-    # original working directory
-    __original_path: str = ""
     # path for the program being profiled
     __program_path: str = ""
     # where we write profile info
     __output_file: str = ""
     # if we output HTML or not
     __html: bool = False
+    # if we profile all code or just target code and code in its child directories
+    __profile_all: bool = False
     # how long between outputting stats during execution
     __output_profile_interval: float = float("inf")
     # when we output the next profile
@@ -520,11 +528,6 @@ class Scalene:
     # Threshold for highlighting lines of code in red.
     __highlight_percentage = 33
 
-    @staticmethod
-    def is_thread_sleeping(tid: int) -> bool:
-        result = Scalene.__is_thread_sleeping[tid]
-        return result
-
     @staticmethod
     def set_thread_sleeping(tid: int) -> None:
         Scalene.__is_thread_sleeping[tid] = True
@@ -659,16 +662,15 @@ process."""
             self.release()
 
     def __init__(self, program_being_profiled: Optional[Filename] = None):
-        if True:
-            # Hijack join.
-            threading.Thread.join = Scalene.thread_join_replacement  # type: ignore
-            # Hijack lock.
-            threading.Lock = Scalene.ReplacementLock  # type: ignore
-            # Hijack system and subprocess calls.
-            # os.system = Scalene.new_os_system  # type: ignore
-            # os.popen = Scalene.new_os_popen  # type: ignore
-            # subprocess.Popen = Scalene.new_subprocess_Popen  # type: ignore
-            pass
+        # Hijack join.
+        threading.Thread.join = Scalene.thread_join_replacement  # type: ignore
+        # Hijack lock.
+        threading.Lock = Scalene.ReplacementLock  # type: ignore
+        # Hijack system and subprocess calls.
+        if Scalene.__hijack_open_calls:
+            os.system = Scalene.new_os_system  # type: ignore
+            os.popen = Scalene.new_os_popen  # type: ignore
+            subprocess.Popen = Scalene.new_subprocess_Popen  # type: ignore
         # Build up signal filenames (adding PID to each).
         Scalene.__malloc_signal_filename = Filename(
             Scalene.__malloc_signal_filename + str(os.getpid())
@@ -763,6 +765,9 @@ process."""
                         python_time / total_frames
                     )
                     Scalene.__cpu_samples_c[fname][lineno] += c_time / total_frames
+                    Scalene.__cpu_samples[fname] += (
+                        python_time + c_time
+                    ) / total_frames
             else:
                 # We can't play the same game here of attributing
                 # time, because we are in a thread, and threads don't
@@ -779,21 +784,11 @@ process."""
                     else:
                         # Not in a call function so we attribute the time to Python.
                         Scalene.__cpu_samples_python[fname][lineno] += normalized_time
+                    Scalene.__cpu_samples[fname] += normalized_time
 
         del new_frames
 
         Scalene.__total_cpu_samples += total_time
-
-        # disabled randomness for now
-        # if False:
-        #    # Pick a random interval, uniformly from m/2 to 3m/2 (so the mean is m)
-        #    mean = Scalene.__mean_signal_interval
-        #    Scalene.__last_signal_interval = random.uniform(mean / 2, mean * 3 / 2)
-        #    signal.setitimer(
-        #        Scalene.cpu_timer_signal,
-        #        Scalene.__last_signal_interval,
-        #        Scalene.__last_signal_interval,
-        #    )
         Scalene.__last_signal_time = Scalene.gettime()
 
     # Returns final frame (up to a line in a file we are profiling), the thread identifier, and the original frame.
@@ -914,7 +909,6 @@ process."""
         before = Scalene.__current_footprint
         for item in arr:
             _alloc_time, action, count, python_fraction = item
-            # print(alloc_time, action, count, python_fraction)
             count /= 1024 * 1024
             is_malloc = action == "M"
             if is_malloc:
@@ -962,6 +956,7 @@ process."""
                     python_frac / allocs
                 ) * (after - before)
                 Scalene.__memory_malloc_count[fname][line_no][bytei] += 1
+                Scalene.__malloc_samples[fname] += 1
                 Scalene.__total_memory_malloc_samples += after - before
             else:
                 Scalene.__memory_free_samples[fname][line_no][bytei] += before - after
@@ -1014,23 +1009,26 @@ process."""
     @lru_cache(None)
     def should_trace(filename: str) -> bool:
         """Return true if the filename is one we should trace."""
-        # Profile anything in the program's directory or a child directory,
-        # but nothing else.
         if filename[0] == "<":
-            return False
-        if "site-packages" in filename or "/usr/lib/python" in filename:
-            # Don't profile Python internals.
+            # Not a real file.
             return False
         if "scalene.py" in filename or "scalene/__main__.py" in filename:
             # Don't profile the profiler.
             return False
+        if Scalene.__profile_all:
+            # Profile everything else.
+            return True
+        if "site-packages" in filename or "/usr/lib/python" in filename:
+            # Don't profile Python internals.
+            return False
+        # Profile anything in the program's directory or a child directory,
+        # but nothing else, unless otherwise specified.
         filename = os.path.abspath(filename)
         return Scalene.__program_path in filename
 
     @staticmethod
     def start() -> None:
         """Initiate profiling."""
-        # os.chdir(Scalene.__program_path)
         Scalene.enable_signals()
         Scalene.__start_time = Scalene.gettime()
 
@@ -1039,18 +1037,6 @@ process."""
         """Complete profiling."""
         Scalene.disable_signals()
         Scalene.__elapsed_time += Scalene.gettime() - Scalene.__start_time
-        # os.chdir(Scalene.__original_path)
-
-    # from https://stackoverflow.com/questions/9836370/fallback-to-stdout-if-no-file-name-provided
-    @staticmethod
-    @contextmanager
-    def file_or_stdout(file_name: Optional[str]) -> Iterator[IO[str]]:
-        """Returns a file handle for writing; if no argument is passed, returns stdout."""
-        if file_name is None:
-            yield sys.stdout
-        else:
-            with open(file_name, "w") as out_file:
-                yield out_file
 
     @staticmethod
     def generate_sparkline(
@@ -1108,24 +1094,12 @@ process."""
         n_malloc_mb = 0.0
         n_python_malloc_mb = 0.0
         n_free_mb = 0.0
-        n_avg_malloc_mb = 0.0
-        n_avg_free_mb = 0.0
-        n_malloc_count = 0
-        n_free_count = 0
         for index in Scalene.__bytei_map[fname][line_no]:
-            # print(fname,line_no,index)
             mallocs = Scalene.__memory_malloc_samples[fname][line_no][index]
             n_malloc_mb += mallocs
             n_python_malloc_mb += Scalene.__memory_python_samples[fname][line_no][index]
-            n_malloc_count += Scalene.__memory_malloc_count[fname][line_no][index]
             frees = Scalene.__memory_free_samples[fname][line_no][index]
             n_free_mb += frees
-            n_free_count += Scalene.__memory_free_count[fname][line_no][index]
-
-            if n_malloc_count > 0:
-                n_avg_malloc_mb += mallocs / n_malloc_count
-            if n_free_count > 0:
-                n_avg_free_mb += frees / n_free_count
 
         n_growth_mb = n_malloc_mb - n_free_mb
         if -1 < n_growth_mb < 0:
@@ -1137,7 +1111,6 @@ process."""
             else n_malloc_mb / Scalene.__total_memory_malloc_samples
         )
         n_python_fraction = 0 if not n_malloc_mb else n_python_malloc_mb / n_malloc_mb
-        # print(n_python_malloc_mb, n_malloc_mb)
         # Finally, print results.
         n_cpu_percent_c_str: str = (
             "" if not n_cpu_percent_c else "%6.1f%%" % n_cpu_percent_c
@@ -1246,7 +1219,6 @@ process."""
             Scalene.__total_memory_free_samples + Scalene.__total_memory_malloc_samples
         ) > 0
         title = Text()
-        # title.append("scalene\n", style="bold")
         mem_usage_line: Union[Text, str] = ""
         if did_sample_memory:
             samples = Scalene.__memory_footprint_samples
@@ -1264,21 +1236,23 @@ process."""
 
         null = open("/dev/null", "w")
         console = Console(width=132, record=True, force_terminal=True, file=null)
+        cpu_percent_threshold = 1
+        if "cpu_percent_threshold" in arguments:
+            cpu_percent_threshold = int(arguments.cpu_percent_threshold)
         for fname in sorted(all_instrumented_files):
-
             fname = Filename(fname)
-            this_cpu_samples = sum(Scalene.__cpu_samples_c[fname].values()) + sum(
-                Scalene.__cpu_samples_python[fname].values()
-            )
-
             try:
-                percent_cpu_time = 100 * this_cpu_samples / Scalene.__total_cpu_samples
+                percent_cpu_time = (
+                    100 * Scalene.__cpu_samples[fname] / Scalene.__total_cpu_samples
+                )
             except ZeroDivisionError:
                 percent_cpu_time = 0
 
-            # Ignore files responsible for less than 1% of execution time,
-            # as long as we aren't profiling memory consumption.
-            if not did_sample_memory and percent_cpu_time < 1:
+            # Ignore files responsible for less than some percent of execution time and fewer than 100 mallocs.
+            if (
+                Scalene.__malloc_samples[fname] < 100
+                and percent_cpu_time < cpu_percent_threshold
+            ):
                 continue
 
             # Print header.
@@ -1341,24 +1315,11 @@ process."""
         """When we exit, disable all signals."""
         Scalene.disable_signals()
 
-    @contextlib.contextmanager
-    @staticmethod
-    def scalene_profiler(_):  # type: ignore
-        """A profiler function, work in progress."""
-        # In principle, this would let people use Scalene as follows:
-        # with scalene_profiler:
-        #   ...
-        profiler = Scalene(None)
-        profiler.start()
-        yield
-        profiler.stop()
-        profiler.output_profiles()  # though this needs to be constrained
-
     @staticmethod
     def main() -> None:
         """Invokes the profiler from the command-line."""
         args, left = parse_args()  # We currently do this twice, but who cares.
-        sys.argv = sys.argv[:1] + left
+        sys.argv = left
         Scalene.set_timer_signal(args.wallclock)
         Scalene.__output_profile_interval = args.profile_interval
         Scalene.__next_output_time = (
@@ -1366,13 +1327,13 @@ process."""
         )
         Scalene.__html = args.html
         Scalene.__output_file = args.outfile
+        Scalene.__profile_all = args.profile_all
         try:
-            with open(args.prog, "rb") as prog_being_profiled:
-                Scalene.__original_path = os.getcwd()
+            with open(sys.argv[0], "rb") as prog_being_profiled:
                 # Read in the code and compile it.
-                code = compile(prog_being_profiled.read(), args.prog, "exec")
+                code = compile(prog_being_profiled.read(), sys.argv[0], "exec")
                 # Push the program's path.
-                program_path = os.path.dirname(os.path.abspath(args.prog))
+                program_path = os.path.dirname(os.path.abspath(sys.argv[0]))
                 sys.path.insert(0, program_path)
                 Scalene.__program_path = program_path
                 # Grab local and global variables.
@@ -1381,9 +1342,9 @@ process."""
                 the_locals = __main__.__dict__
                 the_globals = __main__.__dict__
                 # Splice in the name of the file being executed instead of the profiler.
-                the_globals["__file__"] = os.path.basename(args.prog)
+                the_globals["__file__"] = os.path.basename(sys.argv[0])
                 # Start the profiler.
-                fullname = os.path.join(program_path, os.path.basename(args.prog))
+                fullname = os.path.join(program_path, os.path.basename(sys.argv[0]))
                 profiler = Scalene(Filename(fullname))
                 try:
                     # We exit with this status (returning error code as appropriate).
@@ -1396,7 +1357,8 @@ process."""
                         # Intercept sys.exit and propagate the error code.
                         exit_status = se.code
                     except BaseException:
-                        # print(traceback.format_exc()) # for debugging only
+                        if Scalene.__debug:
+                            print(traceback.format_exc())  # for debugging only
                         pass
                     profiler.stop()
                     # If we've collected any samples, dump them.
@@ -1415,7 +1377,7 @@ process."""
                     print(message)
                     print(traceback.format_exc())
         except (FileNotFoundError, IOError):
-            print("Scalene: could not find input file " + args.prog)
+            print("Scalene: could not find input file " + sys.argv[0])
             sys.exit(-1)
 
 
